%{
/* 
  PURPOSE:    A DDL 2.1 compliant DDL file parser.
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define  LVAL yylval

#include "DDLParserInt.h"
extern int ddlparser_lex();
extern void ddlparser_error(const char*);
%}

%union {
	char *cBuf;
}
%token <cBuf> ITEMNAME_DDL
%token <cBuf> ITEMVALUE_DDL
%token <cBuf> LSITEMVALUE_DDL
%token <cBuf> LOOP_DDL
%token <cBuf> DATABLOCK_DDL
%token <cBuf> UNKNOWN_DDL
%token <cBuf> MISSING_DDL
%token <cBuf> SAVE_BEGIN_DDL
%token <cBuf> SAVE_END_DDL
%type  <cBuf> ItemName ItemValue 


%%

Datablocks :     Sections     /*   Assignments */
             |   Datablocks  Datablock 
             ;

Datablock :    DatablockName  Sections;

Sections:
            | Sections Section

            ;
Section:       Assignment
				 | SaveBegin Assignments SaveEnd
{}
;

Assignments:  /* empty */ 
             | Assignments error Assignment 
             | Assignments Assignment 
{
    ProcessAssignmentsFromDDLParser();
} 
             ;

Assignment:       ItemName ItemValue  /* instance of a non-looped item name value pair */
{
    ProcessOneAssignmentFromDDLParser();
}
  |   ItemNameList ValueList
  ;


ItemNameList:  

  Loop  ItemName           /*  the beginning of a loop_ */
{
    ProcessItemNameListLoopFromDDLParser();
}
 
  | ItemNameList ItemName 
{
    ProcessItemNameListNameFromDDLParser();
}
  ;


ValueList:  ItemValue 
{
    ProcessValueListItemFromDDLParser();
}
  | ValueList ItemValue
{
    ProcessValueListItemFromDDLParser();
}
;

ItemName : ITEMNAME_DDL  
{
   Glob_tBufKeywordSaveDDL = $1;
   ProcessItemNameFromDDLParser();
}
;

Loop :    LOOP_DDL
{
    ProcessLoopFromDDLParser();
}
;

ItemValue:   ITEMVALUE_DDL 
{
   Glob_pBufValueDDL = $1;
   ProcessItemValueFromDDLParser();
}
        | LSITEMVALUE_DDL
{
   Glob_pBufValueDDL = $1;
   ProcessLsItemValueFromDDLParser();
}
        | UNKNOWN_DDL 
{

    ProcessUnknownValueFromDDLParser();
}

        | MISSING_DDL
{

    ProcessMissingValueFromDDLParser();
}
;

SaveBegin: SAVE_BEGIN_DDL
{
    Glob_dataBlockNameDDL = $1;
    ProcessSaveBeginFromDDLParser();
}
;
SaveEnd:   SAVE_END_DDL
{
/* end of save frame is reached 
   _saveobj is deleted, and destructor for Tbl object has CloseFile(0),
   which means that binary file is not created, so the name of that file
   ("./tmp") has no meaning
*/

  Glob_pBufValueDDL = $1;
  ProcessSaveEndFromDDLParser();
}
;

DatablockName: DATABLOCK_DDL
{
  Glob_dataBlockNameDDL = $1;
  ProcessDataBlockNameFromDDLParser();
}
;

%%


